#!/usr/bin/env python
""" Evil Eye: A Face Tracking Eyeball

by Nirav Patel <http://eclecti.cc>

EvilEye.py - Public Domain http://creativecommons.org/licenses/publicdomain/

iris.png - Creative Commons Attribution-Share Alike from:
  http://www.flickr.com/photos/sarahcartwright/369063792/

haarcascade_frontalface_alt.xml - from: http://opencv.willowgarage.com/wiki/
"""

import pygame
from pygame.locals import *
from opencv.cv import *
from opencv.highgui import *

pygame.init()

class EvilEye(object):
  def __init__(self):
    self.size = (848, 480)
    self.display = pygame.display.set_mode(self.size, pygame.FULLSCREEN)
    self.display.fill((0,0,0))
    pygame.display.set_caption('Evil Eye')    

    self.clock = pygame.time.Clock()
    self.cameratick = pygame.time.get_ticks()

    self.storage = cvCreateMemStorage(0)
    self.cascade = cvLoadHaarClassifierCascade("haarcascade_frontalface_alt.xml", cvSize(1,1))
    self.capture = cvCreateCameraCapture(0)
    self.gray = cvCreateImage(cvSize(640,480), 8, 1)

    self.lastrect = None
    self.starty = self.size[1]/2
    self.startx = self.size[0]/2
    self.y = self.starty
    self.x = self.startx
    self.ty = self.y
    self.tx = self.x
    self.vy = 0
    self.vx = 0
    
    self.pupil = pygame.image.load('iris.png')
    self.pupil = self.pupil.convert()
    self.rad = self.pupil.get_width()/2

  def find_face(self):
    frame = cvQueryFrame(self.capture)
    cvCvtColor(frame, self.gray, CV_BGR2GRAY)
    cvEqualizeHist(self.gray,self.gray)
    cvClearMemStorage(self.storage)
    faces = cvHaarDetectObjects(self.gray, self.cascade, self.storage, 1.3, 2, CV_HAAR_DO_CANNY_PRUNING, cvSize(75, 75))
    
    for face in faces:
      self.tx = face.x+face.width/2+(self.size[0]-640)/2
      self.ty = face.y+face.height/2+(self.size[1]-480)/2
      break

  def move_eye(self):
    self.vy = max(-15,min(15,(self.ty-self.y)/10))
    self.vx = max(-15,min(15,(self.tx-self.x)/10))
    
    self.y += int(self.vy)
    self.x += int(self.vx)
    if self.x < self.rad:
      self.x = self.rad
    elif self.x > self.size[0]-self.rad:
      self.x = self.size[0]-self.rad
    if self.y < self.rad:
      self.y = self.rad
    elif self.y > self.size[1]-self.rad:
      self.y = self.size[1]-self.rad

  def composite(self):
    self.display.fill((0,0,0),self.lastrect)
    rect = self.display.blit(self.pupil,(self.x-self.rad,self.y-self.rad))
    pygame.display.update([rect,self.lastrect])
    self.lastrect = rect

  def main(self):
    going = True
    
    while going:
      events = pygame.event.get()
      for e in events:
        if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
          going = False

      now = pygame.time.get_ticks()
      if now > self.cameratick + 100:
        self.find_face()
        self.cameratick = now
      
      self.move_eye()
      self.composite()
      self.clock.tick(30)
      
EvilEye().main()
